@@ -8,13 +8,13 @@ from .exceptions import FieldError
 
 class Field:
     """
-    Field represent mapping between `object` field and representation of
-    this field.
+    ``Field`` represents a mapping between an ``instance`` field and a representation of
+    the field's data.
 
-    :param attribute: A string of either an instance attribute or callable off
-        the object.
+    :param attribute: A string of either an instance attribute or callable of
+        the instance.
 
-    :param column_name: Lets you provide a name for the column that represents
+    :param column_name: An optional column name for the column that represents
         this field in the export.
 
     :param widget: Defines a widget that will be used to represent this
@@ -24,12 +24,16 @@ class Field:
         during import.
 
     :param default: This value will be returned by
-        :meth:`~import_export.fields.Field.clean` if this field's widget did
-        not return an adequate value.
+        :meth:`~import_export.fields.Field.clean` if this field's widget returned
+        a value defined in :attr:`~import_export.fields.empty_values`.
+
+    :param saves_null_values: Controls whether null values are saved on the instance.
+      This can be used if the widget returns null, but there is a default instance
+      value which should not be overwritten.
 
-    :param saves_null_values: Controls whether null values are saved on the object
     :param dehydrate_method: Lets you choose your own method for dehydration rather
         than using `dehydrate_{field_name}` syntax.
+
     :param m2m_add: changes save of this field to add the values, if they do not exist,
         to a ManyToMany field instead of setting all values.  Only useful if field is
         a ManyToMany field.
@@ -69,21 +73,20 @@ class Field:
             return "<%s: %s>" % (path, column_name)
         return "<%s>" % path
 
-    def clean(self, data, **kwargs):
+    def clean(self, row, **kwargs):
         """
         Translates the value stored in the imported datasource to an
         appropriate Python object and returns it.
         """
         try:
-            value = data[self.column_name]
+            value = row[self.column_name]
         except KeyError:
             raise KeyError(
                 "Column '%s' not found in dataset. Available "
-                "columns are: %s" % (self.column_name, list(data))
+                "columns are: %s" % (self.column_name, list(row))
             )
 
-        # If ValueError is raised here, import_obj() will handle it
-        value = self.widget.clean(value, row=data, **kwargs)
+        value = self.widget.clean(value, row=row, **kwargs)
 
         if value in self.empty_values and self.default != NOT_PROVIDED:
             if callable(self.default):
@@ -92,15 +95,15 @@ class Field:
 
         return value
 
-    def get_value(self, obj):
+    def get_value(self, instance):
         """
-        Returns the value of the object's attribute.
+        Returns the value of the instance's attribute.
         """
         if self.attribute is None:
             return None
 
         attrs = self.attribute.split("__")
-        value = obj
+        value = instance
 
         for attr in attrs:
             try:
@@ -118,34 +121,32 @@ class Field:
             value = value()
         return value
 
-    def save(self, obj, data, is_m2m=False, **kwargs):
+    def save(self, instance, row, is_m2m=False, **kwargs):
         """
-        If this field is not declared readonly, the object's attribute will
+        If this field is not declared readonly, the instance's attribute will
         be set to the value returned by :meth:`~import_export.fields.Field.clean`.
         """
         if not self.readonly:
             attrs = self.attribute.split("__")
             for attr in attrs[:-1]:
-                obj = getattr(obj, attr, None)
-            cleaned = self.clean(data, **kwargs)
+                instance = getattr(instance, attr, None)
+            cleaned = self.clean(row, **kwargs)
             if cleaned is not None or self.saves_null_values:
                 if not is_m2m:
-                    setattr(obj, attrs[-1], cleaned)
+                    setattr(instance, attrs[-1], cleaned)
                 else:
                     if self.m2m_add:
-                        getattr(obj, attrs[-1]).add(*cleaned)
+                        getattr(instance, attrs[-1]).add(*cleaned)
                     else:
-                        getattr(obj, attrs[-1]).set(cleaned)
+                        getattr(instance, attrs[-1]).set(cleaned)
 
-    def export(self, obj):
+    def export(self, instance):
         """
-        Returns value from the provided object converted to export
+        Returns value from the provided instance converted to export
         representation.
         """
-        value = self.get_value(obj)
-        if value is None:
-            return ""
-        return self.widget.render(value, obj)
+        value = self.get_value(instance)
+        return self.widget.render(value)
 
     def get_dehydrate_method(self, field_name=None):
         """
