@@ -1,5 +1,6 @@
 import logging
 import warnings
+from warnings import warn
 
 from django.conf import settings
 from django.http import HttpResponse
@@ -7,7 +8,7 @@ from django.utils.timezone import now
 from django.views.generic.edit import FormView
 
 from .formats import base_formats
-from .forms import ExportForm
+from .forms import SelectableFieldsExportForm
 from .resources import modelresource_factory
 from .signals import post_export
 
@@ -15,6 +16,11 @@ logger = logging.getLogger(__name__)
 
 
 class BaseImportExportMixin:
+    """
+    Base mixin for functionality related to importing and exporting via the Admin
+    interface.
+    """
+
     resource_class = None
     resource_classes = []
 
@@ -37,8 +43,12 @@ class BaseImportExportMixin:
                 "subscriptable (list, tuple, ...)"
             )
 
-    def get_resource_classes(self):
-        """Return subscriptable type (list, tuple, ...) containing resource classes"""
+    def get_resource_classes(self, request):
+        """
+        Return subscriptable type (list, tuple, ...) containing resource classes
+        :param request: The request object.
+        :returns: The Resource classes.
+        """
         if self.resource_classes and self.resource_class:
             raise Exception(
                 "Only one of 'resource_class' and 'resource_classes' can be set"
@@ -63,9 +73,23 @@ class BaseImportExportMixin:
         return [self.resource_class]
 
     def get_resource_kwargs(self, request, *args, **kwargs):
-        return {}
+        """
+        Return the kwargs which are to be passed to the Resource constructor.
+        Can be overridden to provide additional kwarg params.
+
+        :param request: The request object.
+        :param kwargs: Keyword arguments.
+        :returns: The Resource kwargs (by default, is the kwargs passed).
+        """
+        return kwargs
 
     def get_resource_index(self, form):
+        """
+        Return the index of the resource class defined in the form.
+
+        :param form: The form object.
+        :returns: The index of the resource as an int.
+        """
         resource_index = 0
         if form and "resource" in form.cleaned_data:
             try:
@@ -76,8 +100,14 @@ class BaseImportExportMixin:
 
 
 class BaseImportMixin(BaseImportExportMixin):
-    def get_import_resource_classes(self):
+    #: If enabled, the import workflow skips the import confirm page
+    #: and imports the data directly.
+    #: See :ref:`import_export_skip_admin_confirm`.
+    skip_import_confirm = False
+
+    def get_import_resource_classes(self, request):
         """
+        :param request: The request object.
         Returns ResourceClass subscriptable (list, tuple, ...) to use for import.
         """
         if hasattr(self, "get_import_resource_class"):
@@ -87,8 +117,7 @@ class BaseImportMixin(BaseImportExportMixin):
                 DeprecationWarning,
             )
             return [self.get_import_resource_class()]
-
-        resource_classes = self.get_resource_classes()
+        resource_classes = self.get_resource_classes(request)
         self.check_resource_classes(resource_classes)
         return resource_classes
 
@@ -98,41 +127,44 @@ class BaseImportMixin(BaseImportExportMixin):
         """
         return [f for f in self.import_formats if f().can_import()]
 
-    def get_import_resource_kwargs(self, request, *args, **kwargs):
-        return self.get_resource_kwargs(request, *args, **kwargs)
+    def get_import_resource_kwargs(self, request, **kwargs):
+        """
+        Returns kwargs which will be passed to the Resource constructor.
+        :param request: The request object.
+        :param kwargs: Keyword arguments.
+        :returns: The kwargs (dict)
+        """
+        return self.get_resource_kwargs(request, **kwargs)
 
-    def choose_import_resource_class(self, form):
+    def choose_import_resource_class(self, form, request):
+        """
+        Identify which class should be used for import
+        :param form: The form object.
+        :param request: The request object.
+        :returns: The export Resource class.
+        """
         resource_index = self.get_resource_index(form)
-        return self.get_import_resource_classes()[resource_index]
+        return self.get_import_resource_classes(request)[resource_index]
+
+    def is_skip_import_confirm_enabled(self):
+        return (
+            getattr(settings, "IMPORT_EXPORT_SKIP_ADMIN_CONFIRM", False)
+            or self.skip_import_confirm is True
+        )
 
 
 class BaseExportMixin(BaseImportExportMixin):
     model = None
-    escape_exported_data = False
-    escape_html = False
-    escape_formulae = False
 
-    @property
-    def should_escape_html(self):
-        if hasattr(settings, "IMPORT_EXPORT_ESCAPE_HTML_ON_EXPORT"):
-            warnings.warn(
-                "IMPORT_EXPORT_ESCAPE_HTML_ON_EXPORT is deprecated and "
-                "will be removed in a future release.",
-                DeprecationWarning,
-            )
-        v = getattr(settings, "IMPORT_EXPORT_ESCAPE_HTML_ON_EXPORT", self.escape_html)
-        if v is True:
-            logger.debug("IMPORT_EXPORT_ESCAPE_HTML_ON_EXPORT is enabled")
-        return v
+    #: If enabled, the export workflow skips the export form and
+    #: exports the data directly.
+    #: See :ref:`import_export_skip_admin_export_ui`.
+    skip_export_form = False
 
-    @property
-    def should_escape_formulae(self):
-        v = getattr(
-            settings, "IMPORT_EXPORT_ESCAPE_FORMULAE_ON_EXPORT", self.escape_formulae
-        )
-        if v is True:
-            logger.debug("IMPORT_EXPORT_ESCAPE_FORMULAE_ON_EXPORT is enabled")
-        return v
+    #: If enabled, the export workflow from Admin UI action menu
+    #: skips the export form and exports the data directly.
+    #: See :ref:`import_export_skip_admin_action_export_ui`.
+    skip_export_form_from_action = False
 
     def get_export_formats(self):
         """
@@ -140,9 +172,11 @@ class BaseExportMixin(BaseImportExportMixin):
         """
         return [f for f in self.export_formats if f().can_export()]
 
-    def get_export_resource_classes(self):
+    def get_export_resource_classes(self, request):
         """
         Returns ResourceClass subscriptable (list, tuple, ...) to use for export.
+        :param request: The request object.
+        :returns: The Resource classes.
         """
         if hasattr(self, "get_export_resource_class"):
             warnings.warn(
@@ -151,26 +185,46 @@ class BaseExportMixin(BaseImportExportMixin):
                 DeprecationWarning,
             )
             return [self.get_export_resource_class()]
-
-        resource_classes = self.get_resource_classes()
+        resource_classes = self.get_resource_classes(request)
         self.check_resource_classes(resource_classes)
         return resource_classes
 
-    def choose_export_resource_class(self, form):
+    def choose_export_resource_class(self, form, request):
+        """
+        Identify which class should be used for export
+        :param request: The request object.
+        :param form: The form object.
+        :returns: The export Resource class.
+        """
         resource_index = self.get_resource_index(form)
-        return self.get_export_resource_classes()[resource_index]
+        return self.get_export_resource_classes(request)[resource_index]
 
-    def get_export_resource_kwargs(self, request, *args, **kwargs):
-        return self.get_resource_kwargs(request, *args, **kwargs)
+    def get_export_resource_kwargs(self, request, **kwargs):
+        """
+        Returns kwargs which will be passed to the Resource constructor.
+        :param request: The request object.
+        :param kwargs: Keyword arguments.
+        :returns: The kwargs (dict)
+        """
+        return self.get_resource_kwargs(request, **kwargs)
 
-    def get_data_for_export(self, request, queryset, *args, **kwargs):
+    def get_export_resource_fields_from_form(self, form):
+        if isinstance(form, SelectableFieldsExportForm):
+            export_fields = form.get_selected_resource_export_fields()
+            if export_fields:
+                return export_fields
+
+        return
+
+    def get_data_for_export(self, request, queryset, **kwargs):
         export_form = kwargs.get("export_form")
-        export_class = self.choose_export_resource_class(export_form)
-        export_resource_kwargs = self.get_export_resource_kwargs(
-            request, *args, **kwargs
-        )
+        export_class = self.choose_export_resource_class(export_form, request)
+        export_resource_kwargs = self.get_export_resource_kwargs(request, **kwargs)
+        export_fields = self.get_export_resource_fields_from_form(export_form)
         cls = export_class(**export_resource_kwargs)
-        export_data = cls.export(*args, queryset=queryset, **kwargs)
+        export_data = cls.export(
+            queryset=queryset, export_fields=export_fields, **kwargs
+        )
         return export_data
 
     def get_export_filename(self, file_format):
@@ -182,15 +236,27 @@ class BaseExportMixin(BaseImportExportMixin):
         )
         return filename
 
+    def is_skip_export_form_enabled(self):
+        return (
+            getattr(settings, "IMPORT_EXPORT_SKIP_ADMIN_EXPORT_UI", False)
+            or self.skip_export_form is True
+        )
+
+    def is_skip_export_form_from_action_enabled(self):
+        return (
+            getattr(settings, "IMPORT_EXPORT_SKIP_ADMIN_ACTION_EXPORT_UI", False)
+            or self.skip_export_form_from_action is True
+        )
+
 
 class ExportViewMixin(BaseExportMixin):
-    form_class = ExportForm
+    form_class = SelectableFieldsExportForm
 
-    def get_export_data(self, file_format, queryset, *args, **kwargs):
+    def get_export_data(self, file_format, queryset, **kwargs):
         """
         Returns file_format representation for given queryset.
         """
-        data = self.get_data_for_export(self.request, queryset, *args, **kwargs)
+        data = self.get_data_for_export(self.request, queryset, **kwargs)
         export_data = file_format.export_data(data)
         return export_data
 
@@ -201,13 +267,20 @@ class ExportViewMixin(BaseExportMixin):
     def get_form_kwargs(self):
         kwargs = super().get_form_kwargs()
         kwargs["formats"] = self.get_export_formats()
+        kwargs["resources"] = self.get_export_resource_classes(self.request)
         return kwargs
 
 
 class ExportViewFormMixin(ExportViewMixin, FormView):
     def form_valid(self, form):
+        warn(
+            "ExportViewFormMixin is deprecated and will be removed "
+            "in a future release",
+            DeprecationWarning,
+            stacklevel=2,
+        )
         formats = self.get_export_formats()
-        file_format = formats[int(form.cleaned_data["file_format"])]()
+        file_format = formats[int(form.cleaned_data["format"])]()
         if hasattr(self, "get_filterset"):
             queryset = self.get_filterset(self.get_filterset_class()).qs
         else:
