@@ -1,5 +1,6 @@
 import os.path
 
+from core.models import Category
 from django.contrib.auth.models import Permission, User
 from django.test.testcases import TestCase
 from django.test.utils import override_settings
@@ -15,8 +16,8 @@ class ImportExportPermissionTest(TestCase):
         self.user = user
         self.client.login(username="admin", password="password")
 
-    def set_user_book_model_permission(self, action):
-        permission = Permission.objects.get(codename="%s_book" % action)
+    def set_user_model_permission(self, action, model_name):
+        permission = Permission.objects.get(codename="%s_%s" % (action, model_name))
         self.user.user_permissions.add(permission)
 
     @override_settings(IMPORT_EXPORT_IMPORT_PERMISSION_CODE="change")
@@ -33,7 +34,7 @@ class ImportExportPermissionTest(TestCase):
 
         with open(filename, "rb") as f:
             data = {
-                "input_format": input_format,
+                "format": input_format,
                 "import_file": f,
             }
 
@@ -44,7 +45,7 @@ class ImportExportPermissionTest(TestCase):
             self.assertEqual(response.status_code, 403)
 
         # user has sufficient permission to import
-        self.set_user_book_model_permission("change")
+        self.set_user_model_permission("change", "book")
 
         response = self.client.get("/admin/core/book/import/")
         self.assertEqual(response.status_code, 200)
@@ -57,7 +58,7 @@ class ImportExportPermissionTest(TestCase):
 
         with open(filename, "rb") as f:
             data = {
-                "input_format": input_format,
+                "format": input_format,
                 "import_file": f,
             }
 
@@ -70,25 +71,39 @@ class ImportExportPermissionTest(TestCase):
             self.assertEqual(response.status_code, 302)
 
     @override_settings(IMPORT_EXPORT_EXPORT_PERMISSION_CODE="change")
-    def test_import_with_permission_set(self):
+    def test_export_with_permission_set(self):
         response = self.client.get("/admin/core/book/export/")
         self.assertEqual(response.status_code, 403)
 
-        data = {"file_format": "0"}
+        data = {"format": "0"}
         response = self.client.post("/admin/core/book/export/", data)
         self.assertEqual(response.status_code, 403)
 
-        self.set_user_book_model_permission("change")
+        self.set_user_model_permission("change", "book")
         response = self.client.get("/admin/core/book/export/")
         self.assertEqual(response.status_code, 200)
 
-        data = {"file_format": "0"}
+        data = {"format": "0"}
         response = self.client.post("/admin/core/book/export/", data)
         self.assertEqual(response.status_code, 200)
 
+    @override_settings(IMPORT_EXPORT_EXPORT_PERMISSION_CODE="change")
+    def test_export_action_with_permission_set(self):
+        self.cat1 = Category.objects.create(name="Cat 1")
+        data = {
+            "action": ["export_admin_action"],
+            "_selected_action": [str(self.cat1.id)],
+        }
+        response = self.client.post("/admin/core/category/", data)
+        self.assertEqual(response.status_code, 403)
+
+        self.set_user_model_permission("change", "category")
+        response = self.client.post("/admin/core/category/", data)
+        self.assertEqual(response.status_code, 200)
+
     @override_settings(IMPORT_EXPORT_EXPORT_PERMISSION_CODE="add")
     def test_check_export_button(self):
-        self.set_user_book_model_permission("change")
+        self.set_user_model_permission("change", "book")
 
         response = self.client.get("/admin/core/book/")
         widget = "import_link"
@@ -98,7 +113,7 @@ class ImportExportPermissionTest(TestCase):
 
     @override_settings(IMPORT_EXPORT_IMPORT_PERMISSION_CODE="add")
     def test_check_import_button(self):
-        self.set_user_book_model_permission("change")
+        self.set_user_model_permission("change", "book")
 
         response = self.client.get("/admin/core/book/")
         widget = "import_link"
