@@ -1,16 +1,18 @@
 import json
 from datetime import date, datetime, time, timedelta
 from decimal import Decimal
-from unittest import mock, skipUnless
+from unittest import mock
+from unittest.mock import patch
 
 import django
-import pytz
 from core.models import Author, Book, Category
+from core.tests.utils import ignore_utcnow_deprecation_warning
 from django.test import TestCase
 from django.test.utils import override_settings
 from django.utils import timezone
 
 from import_export import widgets
+from import_export.exceptions import WidgetError
 
 
 class WidgetTest(TestCase):
@@ -24,7 +26,17 @@ class WidgetTest(TestCase):
         self.assertEqual("1", self.widget.render(1))
 
 
-class CharWidgetTest(TestCase):
+class RowDeprecationTestMixin(object):
+    def test_render_row_deprecation(self):
+        with self.assertWarnsRegex(
+            DeprecationWarning,
+            r"^The 'obj' parameter is deprecated and "
+            "will be removed in a future release$",
+        ):
+            self.widget.render(Book.objects.none(), obj={"a": 1})
+
+
+class CharWidgetTest(TestCase, RowDeprecationTestMixin):
     def setUp(self):
         self.widget = widgets.CharWidget()
 
@@ -34,27 +46,20 @@ class CharWidgetTest(TestCase):
     def test_render(self):
         self.assertEqual("1", self.widget.render(1))
 
-    def test_clean_coerce_to_string(self):
-        self.widget = widgets.CharWidget(coerce_to_string=True)
-        self.assertEqual("1", self.widget.clean(1))
+    def test_render_no_coerce_to_string(self):
+        self.widget = widgets.CharWidget(coerce_to_string=False)
+        self.assertEqual(1, self.widget.render(1))
 
-    def test_clean_no_coerce_to_string(self):
-        self.assertEqual(1, self.widget.clean(1))
-
-    def test_clean_coerce_to_string_None(self):
-        self.widget = widgets.CharWidget(coerce_to_string=True)
+    def test_clean_with_allow_blank_is_False(self):
+        self.widget = widgets.CharWidget(allow_blank=False)
         self.assertIsNone(self.widget.clean(None))
 
-    def test_clean_coerce_to_string_with_allow_blank(self):
-        self.widget = widgets.CharWidget(coerce_to_string=True, allow_blank=True)
+    def test_clean_with_allow_blank_is_True(self):
+        self.widget = widgets.CharWidget(allow_blank=True)
         self.assertEqual("", self.widget.clean(None))
 
-    def test_clean_coerce_to_string_is_False_with_allow_blank(self):
-        self.widget = widgets.CharWidget(coerce_to_string=False, allow_blank=True)
-        self.assertIsNone(self.widget.clean(None))
 
-
-class BooleanWidgetTest(TestCase):
+class BooleanWidgetTest(TestCase, RowDeprecationTestMixin):
     def setUp(self):
         self.widget = widgets.BooleanWidget()
 
@@ -83,30 +88,28 @@ class BooleanWidgetTest(TestCase):
         self.assertEqual(self.widget.render(False), "0")
         self.assertEqual(self.widget.render(None), "")
 
+    def test_render_coerce_to_string_is_False(self):
+        self.widget = widgets.BooleanWidget(coerce_to_string=False)
+        self.assertTrue(self.widget.render(True))
+        self.assertFalse(self.widget.render(False))
+        self.assertIsNone(self.widget.render(None))
+
+    def test_render_invalid_type(self):
+        self.assertEqual(self.widget.render("a"), "")
+
 
 class FormatDatetimeTest(TestCase):
     date = date(10, 8, 2)
     target_dt = "02.08.0010"
     format = "%d.%m.%Y"
 
-    @skipUnless(
-        django.VERSION[0] < 4, f"skipping django {django.VERSION} version specific test"
-    )
-    def test_format_datetime_lt_django4(self):
-        self.assertEqual(
-            self.target_dt, widgets.format_datetime(self.date, self.format)
-        )
-
-    @skipUnless(
-        django.VERSION[0] >= 4, f"running django {django.VERSION} version specific test"
-    )
     def test_format_datetime_gte_django4(self):
         self.assertEqual(
             self.target_dt, widgets.format_datetime(self.date, self.format)
         )
 
 
-class DateWidgetTest(TestCase):
+class DateWidgetTest(TestCase, RowDeprecationTestMixin):
     def setUp(self):
         self.date = date(2012, 8, 13)
         self.widget = widgets.DateWidget("%d.%m.%Y")
@@ -117,6 +120,13 @@ class DateWidgetTest(TestCase):
     def test_render_none(self):
         self.assertEqual(self.widget.render(None), "")
 
+    def test_render_invalid_type(self):
+        self.assertEqual(self.widget.render(int(1)), "")
+
+    def test_render_coerce_to_string_is_False(self):
+        self.widget = widgets.DateWidget(coerce_to_string=False)
+        self.assertEqual(self.date, self.widget.render(self.date))
+
     def test_render_datetime_safe(self):
         """datetime_safe is supposed to be used to support dates older than 1000"""
         self.date = date(10, 8, 2)
@@ -131,10 +141,16 @@ class DateWidgetTest(TestCase):
     def test_clean_returns_date_when_date_passed(self):
         self.assertEqual(self.date, self.widget.clean(self.date))
 
-    def test_clean_raises_ValueError(self):
+    @patch("import_export.widgets.logger")
+    def test_clean_raises_ValueError(self, mock_logger):
         self.widget = widgets.DateWidget("x")
-        with self.assertRaisesRegex(ValueError, "Enter a valid date."):
+        with self.assertRaisesRegex(
+            ValueError, "Value could not be parsed using defined formats."
+        ):
             self.widget.clean("2021-05-01")
+        mock_logger.debug.assert_called_with(
+            "time data '2021-05-01' does not match format 'x'"
+        )
 
     @override_settings(USE_TZ=True)
     def test_use_tz(self):
@@ -147,7 +163,7 @@ class DateWidgetTest(TestCase):
         self.assertEqual(("%Y-%m-%d",), self.widget.formats)
 
 
-class DateTimeWidgetTest(TestCase):
+class DateTimeWidgetTest(TestCase, RowDeprecationTestMixin):
     def setUp(self):
         self.datetime = datetime(2012, 8, 13, 18, 0, 0)
         self.widget = widgets.DateTimeWidget("%d.%m.%Y %H:%M:%S")
@@ -158,17 +174,41 @@ class DateTimeWidgetTest(TestCase):
     def test_render_none(self):
         self.assertEqual(self.widget.render(None), "")
 
+    def test_render_invalid_type(self):
+        self.assertEqual(self.widget.render(int(1)), "")
+
+    def test_render_coerce_to_string_is_False(self):
+        self.widget = widgets.DateTimeWidget(coerce_to_string=False)
+        self.assertEqual(self.datetime, self.widget.render(self.datetime))
+
     def test_clean(self):
         self.assertEqual(self.widget.clean("13.08.2012 18:00:00"), self.datetime)
 
+    @patch("import_export.widgets.logger")
+    def test_clean_raises_ValueError(self, mock_logger):
+        self.widget = widgets.DateTimeWidget("x")
+        with self.assertRaisesRegex(
+            ValueError, "Value could not be parsed using defined formats."
+        ):
+            self.widget.clean("2021-05-01")
+        mock_logger.debug.assert_called_with(
+            "time data '2021-05-01' does not match format 'x'"
+        )
+
+    @ignore_utcnow_deprecation_warning
     @override_settings(USE_TZ=True, TIME_ZONE="Europe/Ljubljana")
     def test_use_tz(self):
+        import pytz
+
         utc_dt = timezone.make_aware(self.datetime, pytz.UTC)
         self.assertEqual(self.widget.render(utc_dt), "13.08.2012 20:00:00")
         self.assertEqual(self.widget.clean("13.08.2012 20:00:00"), utc_dt)
 
+    @ignore_utcnow_deprecation_warning
     @override_settings(USE_TZ=True, TIME_ZONE="Europe/Ljubljana")
     def test_clean_returns_tz_aware_datetime_when_naive_datetime_passed(self):
+        import pytz
+
         # issue 1165
         if django.VERSION >= (5, 0):
             from zoneinfo import ZoneInfo
@@ -179,8 +219,11 @@ class DateTimeWidgetTest(TestCase):
         target_dt = timezone.make_aware(self.datetime, tz)
         self.assertEqual(target_dt, self.widget.clean(self.datetime))
 
+    @ignore_utcnow_deprecation_warning
     @override_settings(USE_TZ=True, TIME_ZONE="Europe/Ljubljana")
     def test_clean_handles_tz_aware_datetime(self):
+        import pytz
+
         self.datetime = datetime(2012, 8, 13, 18, 0, 0, tzinfo=pytz.timezone("UTC"))
         self.assertEqual(self.datetime, self.widget.clean(self.datetime))
 
@@ -224,7 +267,7 @@ class DateTimeWidgetBefore1900Test(TestCase):
         self.assertEqual(self.datetime, self.widget.clean("13.08.1868"))
 
 
-class TimeWidgetTest(TestCase):
+class TimeWidgetTest(TestCase, RowDeprecationTestMixin):
     def setUp(self):
         self.time = time(20, 15, 0)
         self.widget = widgets.TimeWidget("%H:%M:%S")
@@ -235,6 +278,13 @@ class TimeWidgetTest(TestCase):
     def test_render_none(self):
         self.assertEqual(self.widget.render(None), "")
 
+    def test_render_invalid_type(self):
+        self.assertEqual(self.widget.render(int(1)), "")
+
+    def test_render_coerce_to_string_is_False(self):
+        self.widget = widgets.TimeWidget(coerce_to_string=False)
+        self.assertEqual(self.time, self.widget.render(self.time))
+
     def test_clean(self):
         self.assertEqual(self.widget.clean("20:15:00"), self.time)
 
@@ -243,16 +293,22 @@ class TimeWidgetTest(TestCase):
         self.widget = widgets.TimeWidget()
         self.assertEqual(("%H:%M:%S",), self.widget.formats)
 
-    def test_clean_raises_ValueError(self):
+    @patch("import_export.widgets.logger")
+    def test_clean_raises_ValueError(self, mock_logger):
         self.widget = widgets.TimeWidget("x")
-        with self.assertRaisesRegex(ValueError, "Enter a valid time."):
+        with self.assertRaisesRegex(
+            ValueError, "Value could not be parsed using defined formats."
+        ):
             self.widget.clean("20:15:00")
+        mock_logger.debug.assert_called_with(
+            "time data '20:15:00' does not match format 'x'"
+        )
 
     def test_clean_returns_time_when_time_passed(self):
         self.assertEqual(self.time, self.widget.clean(self.time))
 
 
-class DurationWidgetTest(TestCase):
+class DurationWidgetTest(TestCase, RowDeprecationTestMixin):
     def setUp(self):
         self.duration = timedelta(hours=1, minutes=57, seconds=0)
         self.widget = widgets.DurationWidget()
@@ -266,6 +322,13 @@ class DurationWidgetTest(TestCase):
     def test_render_zero(self):
         self.assertEqual(self.widget.render(timedelta(0)), "0:00:00")
 
+    def test_render_coerce_to_string_is_False(self):
+        self.widget = widgets.DurationWidget(coerce_to_string=False)
+        self.assertEqual(self.duration, self.widget.render(self.duration))
+
+    def test_render_invalid_type(self):
+        self.assertEqual(self.widget.render(int(1)), "")
+
     def test_clean(self):
         self.assertEqual(self.widget.clean("1:57:00"), self.duration)
 
@@ -275,13 +338,15 @@ class DurationWidgetTest(TestCase):
     def test_clean_zero(self):
         self.assertEqual(self.widget.clean("0:00:00"), timedelta(0))
 
-    @mock.patch("import_export.widgets.parse_duration", side_effect=ValueError("err"))
-    def test_clean_raises_ValueError(self, _):
-        with self.assertRaisesRegex(ValueError, "Enter a valid duration."):
+    @patch("import_export.widgets.parse_duration", side_effect=ValueError("err"))
+    @patch("import_export.widgets.logger")
+    def test_clean_raises_ValueError(self, mock_logger, _):
+        with self.assertRaisesRegex(ValueError, "Value could not be parsed."):
             self.widget.clean("x")
+        mock_logger.debug.assert_called_with("err")
 
 
-class NumberWidgetTest(TestCase):
+class NumberWidgetTest(TestCase, RowDeprecationTestMixin):
     def setUp(self):
         self.value = 11.111
         self.widget = widgets.NumberWidget()
@@ -300,22 +365,14 @@ class NumberWidgetTest(TestCase):
         self.assertFalse(self.widget.is_empty(0))
 
     def test_render(self):
-        self.assertEqual(self.value, self.widget.render(self.value))
+        self.assertEqual("11.111", self.widget.render(self.value))
 
     def test_render_None_coerce_to_string_False(self):
-        self.assertIsNone(self.widget.render(None))
+        self.assertEqual("", self.widget.render(None))
 
-    @skipUnless(
-        django.VERSION[0] < 4, f"skipping django {django.VERSION} version specific test"
-    )
-    @override_settings(LANGUAGE_CODE="fr-fr", USE_L10N=True)
-    def test_locale_render_coerce_to_string_lt4(self):
-        self.assertEqual("11,111", self.widget_coerce_to_string.render(self.value))
+    def test_render_invalid_type(self):
+        self.assertEqual(self.widget.render("a"), "")
 
-    @skipUnless(
-        django.VERSION[0] >= 4,
-        f"skipping django {django.VERSION} version specific test",
-    )
     @override_settings(LANGUAGE_CODE="fr-fr")
     def test_locale_render_coerce_to_string_gte4(self):
         self.assertEqual("11,111", self.widget_coerce_to_string.render(self.value))
@@ -324,7 +381,7 @@ class NumberWidgetTest(TestCase):
         self.assertEqual("", self.widget_coerce_to_string.render(None))
 
 
-class FloatWidgetTest(TestCase):
+class FloatWidgetTest(TestCase, RowDeprecationTestMixin):
     def setUp(self):
         self.value = 11.111
         self.widget = widgets.FloatWidget()
@@ -334,7 +391,10 @@ class FloatWidgetTest(TestCase):
         self.assertEqual(self.widget.clean(11.111), self.value)
 
     def test_render(self):
-        self.assertEqual(self.widget.render(self.value), self.value)
+        self.assertEqual(self.widget.render(self.value), "11.111")
+
+    def test_render_invalid_type(self):
+        self.assertEqual(self.widget.render("a"), "")
 
     def test_clean_string_zero(self):
         self.assertEqual(self.widget.clean("0"), 0.0)
@@ -345,35 +405,30 @@ class FloatWidgetTest(TestCase):
         self.assertEqual(self.widget.clean(" "), None)
         self.assertEqual(self.widget.clean("\r\n\t"), None)
 
-    @skipUnless(
-        django.VERSION[0] < 4, f"skipping django {django.VERSION} version specific test"
-    )
-    @override_settings(LANGUAGE_CODE="fr-fr", USE_L10N=True)
-    def test_locale_render_coerce_to_string_lt4(self):
-        self.assertEqual(self.widget_coerce_to_string.render(self.value), "11,111")
-
-    @skipUnless(
-        django.VERSION[0] >= 4,
-        f"skipping django {django.VERSION} version specific test",
-    )
     @override_settings(LANGUAGE_CODE="fr-fr")
     def test_locale_render_coerce_to_string_gte4(self):
         self.assertEqual(self.widget_coerce_to_string.render(self.value), "11,111")
 
 
-class DecimalWidgetTest(TestCase):
+class DecimalWidgetTest(TestCase, RowDeprecationTestMixin):
     def setUp(self):
         self.value = Decimal("11.111")
         self.widget = widgets.DecimalWidget()
-        self.widget_coerce_to_string = widgets.DecimalWidget(coerce_to_string=True)
 
     def test_clean(self):
         self.assertEqual(self.widget.clean("11.111"), self.value)
         self.assertEqual(self.widget.clean(11.111), self.value)
 
-    def test_render(self):
+    def test_render_coerce_to_string_is_False(self):
+        self.widget = widgets.DecimalWidget(coerce_to_string=False)
         self.assertEqual(self.widget.render(self.value), self.value)
 
+    def test_render(self):
+        self.assertEqual(self.widget.render(self.value), "11.111")
+
+    def test_render_invalid_type(self):
+        self.assertEqual(self.widget.render("1"), "")
+
     def test_clean_string_zero(self):
         self.assertEqual(self.widget.clean("0"), Decimal("0"))
         self.assertEqual(self.widget.clean("0.0"), Decimal("0"))
@@ -383,23 +438,12 @@ class DecimalWidgetTest(TestCase):
         self.assertEqual(self.widget.clean(" "), None)
         self.assertEqual(self.widget.clean("\r\n\t"), None)
 
-    @skipUnless(
-        django.VERSION[0] < 4, f"skipping django {django.VERSION} version specific test"
-    )
-    @override_settings(LANGUAGE_CODE="fr-fr", USE_L10N=True)
-    def test_locale_render_coerce_to_string_lt4(self):
-        self.assertEqual(self.widget_coerce_to_string.render(self.value), "11,111")
-
-    @skipUnless(
-        django.VERSION[0] >= 4,
-        f"skipping django {django.VERSION} version specific test",
-    )
     @override_settings(LANGUAGE_CODE="fr-fr")
     def test_locale_render_coerce_to_string_gte4(self):
-        self.assertEqual(self.widget_coerce_to_string.render(self.value), "11,111")
+        self.assertEqual(self.widget.render(self.value), "11,111")
 
 
-class IntegerWidgetTest(TestCase):
+class IntegerWidgetTest(TestCase, RowDeprecationTestMixin):
     def setUp(self):
         self.value = 0
         self.widget = widgets.IntegerWidget()
@@ -421,23 +465,15 @@ class IntegerWidgetTest(TestCase):
         self.assertEqual(self.widget.clean(" "), None)
         self.assertEqual(self.widget.clean("\n\t\r"), None)
 
-    @skipUnless(
-        django.VERSION[0] < 4, f"skipping django {django.VERSION} version specific test"
-    )
-    @override_settings(LANGUAGE_CODE="fr-fr", USE_L10N=True)
-    def test_locale_render_lt_django4(self):
-        self.assertEqual(self.widget_coerce_to_string.render(self.value), "0")
+    def test_render_invalid_type(self):
+        self.assertEqual(self.widget.render("a"), "")
 
-    @skipUnless(
-        django.VERSION[0] >= 4,
-        f"skipping django {django.VERSION} version specific test",
-    )
     @override_settings(LANGUAGE_CODE="fr-fr")
     def test_locale_render_gte_django4(self):
         self.assertEqual(self.widget_coerce_to_string.render(self.value), "0")
 
 
-class ForeignKeyWidgetTest(TestCase):
+class ForeignKeyWidgetTest(TestCase, RowDeprecationTestMixin):
     def setUp(self):
         self.widget = widgets.ForeignKeyWidget(Author)
         self.natural_key_author_widget = widgets.ForeignKeyWidget(
@@ -483,6 +519,19 @@ class ForeignKeyWidgetTest(TestCase):
         with self.assertRaises(TypeError):
             birthday_widget.clean("Foo", row=row_dict, row_number=1)
 
+    def test_lookup_multiple_columns(self):
+        # issue 1516 - override the values used to lookup an entry
+        class BirthdayWidget(widgets.ForeignKeyWidget):
+            def get_lookup_kwargs(self, value, row, *args, **kwargs):
+                return {"name": row["name"], "birthday": row["birthday"]}
+
+        target_author = Author.objects.create(name="James Joyce", birthday="1882-02-02")
+        row_dict = {"name": "James Joyce", "birthday": "1882-02-02"}
+        birthday_widget = BirthdayWidget(Author, "name")
+        # prove that the overridden kwargs identify a row
+        res = birthday_widget.clean("non-existent name", row=row_dict)
+        self.assertEqual(target_author, res)
+
     def test_render_handles_value_error(self):
         class TestObj(object):
             @property
@@ -534,8 +583,18 @@ class ForeignKeyWidgetTest(TestCase):
             json.dumps(self.book.natural_key()),
         )
 
+    def test_natural_foreign_key_with_key_is_id(self):
+        with self.assertRaises(WidgetError) as e:
+            widgets.ForeignKeyWidget(
+                Author, use_natural_foreign_keys=True, key_is_id=True
+            )
+        self.assertEqual(
+            "use_natural_foreign_keys and key_is_id " "cannot both be True",
+            str(e.exception),
+        )
 
-class ManyToManyWidget(TestCase):
+
+class ManyToManyWidget(TestCase, RowDeprecationTestMixin):
     def setUp(self):
         self.widget = widgets.ManyToManyWidget(Category)
         self.widget_name = widgets.ManyToManyWidget(Category, field="name")
@@ -596,8 +655,11 @@ class ManyToManyWidget(TestCase):
             "%s,%s" % (self.cat1.name, self.cat2.name),
         )
 
+    def test_render_value_none_as_blank(self):
+        self.assertEqual("", self.widget.render(None))
 
-class JSONWidgetTest(TestCase):
+
+class JSONWidgetTest(TestCase, RowDeprecationTestMixin):
     def setUp(self):
         self.value = {"value": 23}
         self.widget = widgets.JSONWidget()
@@ -622,7 +684,7 @@ class JSONWidgetTest(TestCase):
         self.assertEqual(self.widget.render({"value": None}), '{"value": null}')
 
 
-class SimpleArrayWidgetTest(TestCase):
+class SimpleArrayWidgetTest(TestCase, RowDeprecationTestMixin):
     def setUp(self):
         self.value = {"value": 23}
         self.widget = widgets.SimpleArrayWidget()
@@ -646,3 +708,18 @@ class SimpleArrayWidgetTest(TestCase):
         v = ["a", "b", "c"]
         s = "a,b,c"
         self.assertEqual(s, self.widget.render(v))
+
+    def test_render_no_coerce_to_string(self):
+        v = [1, 2, 3]
+        self.widget = widgets.SimpleArrayWidget(coerce_to_string=False)
+        self.assertEqual(v, self.widget.render(v))
+
+    def test_render_none_coerce_to_string_is_True(self):
+        self.widget = widgets.SimpleArrayWidget()
+        self.assertTrue(self.widget.coerce_to_string)
+        self.assertEqual("", self.widget.render(None))
+
+    def test_render_none_coerce_to_string_is_False(self):
+        self.widget = widgets.SimpleArrayWidget(coerce_to_string=False)
+        self.assertFalse(self.widget.coerce_to_string)
+        self.assertIsNone(self.widget.render(None))
